local idTweaks = LibStub('AceAddon-3.0'):GetAddon('idTweaks')
local MailModule = idTweaks:NewModule('Mail', 'AceConsole-3.0', 'AceEvent-3.0')

MailModule.DEBUG = false

function MailModule:Debug(...)
  if MailModule.DEBUG then
    self:Print(...)
  end
end

function MailModule:setDefaults()
  self:Debug('MailModule:setDefaults')

  self.defaults = {
    profile = {
      enabled = true,
      autoSendMail = true
    }
  }
end

function MailModule:setDatabase()
  self:Debug('MailModule:setDatabase')

  self.db = idTweaks.db:RegisterNamespace('Mail', self.defaults)
end

function MailModule:setOptions()
  self:Debug('MailModule:setOptions')

  self.options = {
    name = 'Mail',
    handler = self,
    type = 'group',
    args = {
      enabled = {
        order = 1,
        name = 'Enabled',
        desc = 'Enables / Disables the Module',
        handler = self,
        type = 'toggle',
        get = function()
          return self.db.profile.enabled
        end,
        set = function(_, value)
          self.db.profile.enabled = value
          self:check()
        end,
      },

      autoSendMail = {
        name = 'Auto Send',
        desc = 'When the mail has a Recipient and all item slots are filled, send it.',
        handler = self,
        type = 'toggle',
        disabled = function()
          return not self.db.profile.enabled
        end,
        get = function()
          return self.db.profile.autoSendMail
        end,
        set = function(_, value)
          self.db.profile.autoSendMail = value
        end
      }
    },
  }
end

function MailModule:registerOptions()
  self:Debug('MailModule:registerOptions')

  LibStub('AceConfig-3.0'):RegisterOptionsTable('Mail', self.options)
  LibStub('AceConfigDialog-3.0'):AddToBlizOptions('Mail', 'Mail', 'idTweaks')
end

function MailModule:OnInitialize()
  self:Debug('MailModule:OnInitialize')

  self:setDefaults()
  self:setDatabase()
  self:setOptions()
end

function MailModule:OnEnable()
  self:Debug('MailModule:OnEnable')

  self:registerOptions()

  self:check()
end

function MailModule:OnDisable()
  self:Debug('MailModule:OnDisable')

  self:disable()
end

function MailModule:check()
  self:Debug('MailModule:check')

  if self.db.profile.enabled then
    self:enable()
  else
    self:disable()
  end
end

function MailModule:enable()
  self:Debug('MailModule:enable')

  self:RegisterEvent('MAIL_SEND_INFO_UPDATE')
end

function MailModule:disable()
  self:Debug('MailModule:disable')

  self:UnregisterEvent('MAIL_SEND_INFO_UPDATE')
end

-- Gets the number of slots that are free in a mail.
function MailModule:getFreeMailSlots()
  self:Debug('MailModule:getFreeMailSlots')

  local freeSlots = 0

  for slot = 1, 12 do
    itemName, _, _, _ = GetSendMailItem(slot)

    if not itemName then
      freeSlots = freeSlots + 1
    end
  end

  return freeSlots
end

-- Returns true if the mail has all items attached.
function MailModule:mailIsFull()
  self:Debug('MailModule:mailIsFull')

  return self:getFreeMailSlots() == 0
end

-- Gets the recipient from the mail editbox.
function MailModule:getMailRecipient()
  self:Debug('MailModule:getMailRecipient')

  return SendMailNameEditBox:GetText()
end

-- Sets the recipient in the mail editbox.
function MailModule:setMailRecipient(recipient)
  self:Debug('MailModule:setMailRecipient')

  SendMailNameEditBox:SetText(recipient)
end

-- Returns true if the mail has a recipient.
function MailModule:mailHasRecipient()
  self:Debug('MailModule:mailHasRecipient')

  return self:getMailRecipient() ~= ''
end

-- Gets the subject from the mail editbox.
function MailModule:getMailSubject()
  self:Debug('MailModule:getMailSubject')

  return SendMailSubjectEditBox:GetText()
end

-- Returns true if the mail has a recipient.
function MailModule:mailHasSubject()
  self:Debug('MailModule:mailHasSubject')

  return self:getMailSubject() ~= ''
end

-- Gets the body from the mail editbox.
function MailModule:getMailBody()
  self:Debug('MailModule:getMailBody')

  return SendMailScrollChildFrame:GetText()
end

-- Returns true if the mail can be sent.
function MailModule:canSendMail()
  self:Debug('MailModule:canSendMail')

  return self:mailHasRecipient() and self:mailHasSubject() and self:mailIsFull()
end

-- Sends the mail.
function MailModule:sendMail()
  self:Debug('MailModule:sendMail')

  local recipient = self:getMailRecipient()
  local subject = self:getMailSubject()
  -- local body = self:getMailBody()

  SendMail(recipient, subject, '')

  self:setMailRecipient(recipient)
end

function MailModule:MAIL_SEND_INFO_UPDATE()
  self:Debug('MailModule:MAIL_SEND_INFO_UPDATE')

  if self.db.profile.autoSendMail == false then
    return
  end

  if self:canSendMail() then
    self:sendMail()
  end
end
