local idTweaks = LibStub('AceAddon-3.0'):GetAddon('idTweaks')
local QuestingModule = idTweaks:NewModule('Questing', 'AceConsole-3.0', 'AceEvent-3.0')

QuestingModule.DEBUG = false

function QuestingModule:Debug(...)
  if QuestingModule.DEBUG then
    self:Print(...)
  end
end

function QuestingModule:setDefaults()
  self:Debug('QuestingModule:setDefaults')

  self.defaults = {
    profile = {
      enabled = true,

      pauseKey = 'shift',
      acceptQuests = true,
      completeQuests = true,
    }
  }
end

function QuestingModule:setDatabase()
  self:Debug('QuestingModule:setDatabase')

  self.db = idTweaks.db:RegisterNamespace('Questing', self.defaults)
end

function QuestingModule:setOptions()
  self:Debug('QuestingModule:setOptions')

  self.options = {
    name = 'Questing',
    desc = 'Options for questing.',
    handler = self,
    type = 'group',
    args = {
      enabled = {
        order = 1,
        name = 'Enabled',
        desc = 'Enables / Disables the Module',
        handler = self,
        type = 'toggle',
        get = function()
          return self.db.profile.enabled
        end,
        set = function(_, value)
          self.db.profile.enabled = value
          self:check()
        end,
      },

      pauseKey = {
        name = 'Pause Key',
        desc = 'While you hold this key, nothing will happen, so you can read your quest in peace.',
        handler = self,
        type = 'select',
        values = {
          shift = 'Shift',
          control = 'Control',
          alt = 'Alt',
        },
        disabled = function()
          return not self.db.profile.enabled
        end,
        get = function()
          return self.db.profile.pauseKey
        end,
        set = function(_, value)
          self.db.profile.pauseKey = value
        end
      },

      acceptQuests = {
        name = 'Accept Quests',
        desc = 'Automatically accept all quests. Might need a few clicks to get them all.',
        handler = self,
        type = 'toggle',
        disabled = function()
          return not self.db.profile.enabled
        end,
        get = function()
          return self.db.profile.acceptQuests
        end,
        set = function(_, value)
          self.db.profile.acceptQuests = value
        end
      },

      completeQuests = {
        name = 'Complete quests',
        desc = 'Automatically complete all quests that dont require you to pick a reward.',
        handler = self,
        type = 'toggle',
        disabled = function()
          return not self.db.profile.enabled
        end,
        get = function()
          return self.db.profile.completeQuests
        end,
        set = function(_, value)
          self.db.profile.completeQuests = value
        end
      }
    },
  }
end

function QuestingModule:registerOptions()
  self:Debug('QuestingModule:registerOptions')

  LibStub('AceConfig-3.0'):RegisterOptionsTable('Questing', self.options)
  LibStub('AceConfigDialog-3.0'):AddToBlizOptions('Questing', 'Questing', 'idTweaks')
end

function QuestingModule:OnInitialize()
  self:Debug('QuestingModule:OnInitialize')

  self:setDefaults()
  self:setDatabase()
  self:setOptions()
end

function QuestingModule:OnEnable()
  self:Debug('QuestingModule:OnEnable')

  self:registerOptions()

  self:check()
end

function QuestingModule:OnDisable()
  self:Debug('QuestingModule:OnDisable')

  self:disable()
end

function QuestingModule:check()
  self:Debug('QuestingModule:check')

  if self.db.profile.enabled then
    self:enable()
  else
    self:disable()
  end
end

function QuestingModule:enable()
  self:Debug('QuestingModule:enable')

  self:RegisterEvent('GOSSIP_SHOW')
  self:RegisterEvent('QUEST_AUTOCOMPLETE')
  self:RegisterEvent('QUEST_COMPLETE')
  self:RegisterEvent('QUEST_DETAIL')
  self:RegisterEvent('QUEST_GREETING')
  self:RegisterEvent('QUEST_LOG_UPDATE')
  self:RegisterEvent('QUEST_PROGRESS')
end

function QuestingModule:disable()
  self:Debug('QuestingModule:disable')

  self:UnregisterEvent('GOSSIP_SHOW')
  self:UnregisterEvent('QUEST_AUTOCOMPLETE')
  self:UnregisterEvent('QUEST_COMPLETE')
  self:UnregisterEvent('QUEST_DETAIL')
  self:UnregisterEvent('QUEST_GREETING')
  self:UnregisterEvent('QUEST_LOG_UPDATE')
  self:UnregisterEvent('QUEST_PROGRESS')
end

function QuestingModule:shouldAutomate()
  self:Debug('QuestingModule:shouldAutomate')

  local output = true
  local key = self.db.profile.pauseKey

  if key == 'shift' then
    if IsShiftKeyDown() then
      output = false
    end
  end

  if key == 'control' then
    if IsControlKeyDown() then
      output = false
    end
  end

  if key == 'alt' then
    if IsAltKeyDown() then
      output = false
    end
  end

  self:Debug('QuestingModule:shouldAutomate:output', output)

  return output
end

function QuestingModule:shouldAccept()
  self:Debug('QuestingModule:shouldAccept')

  local output = not not self.db.profile.acceptQuests

  self:Debug('QuestingModule:shouldAccept:output', output)

  return output
end

function QuestingModule:shouldComplete()
  self:Debug('QuestingModule:shouldComplete')

  local output = not not self.db.profile.completeQuests

  self:Debug('QuestingModule:shouldComplete:output', output)

  return output
end

function QuestingModule:stripText(text)
  if not text then return end
  text = text:gsub('|c%x%x%x%x%x%x%x%x(.-)|r','%1')
  text = text:gsub('%[.*%]%s*','')
  text = text:gsub('(.+) %(.+%)', '%1')
  text = text:trim()
  return text
end

function QuestingModule:QUEST_AUTOCOMPLETE(event, questId)
  self:Debug('QuestingModule:QUEST_AUTOCOMPLETE')

  if not self:shouldAutomate() then return end
  if not self:shouldComplete() then return end
  for i = 0, 6 do
    local id = GetQuestIndexForWatch(i)
    if id then
      ShowQuestComplete(id)
    end
  end
end

function QuestingModule:QUEST_COMPLETE(event)
  self:Debug('QuestingModule:QUEST_COMPLETE')

  if not self:shouldAutomate() then return end
  if not self:shouldComplete() then return end
  if GetNumQuestChoices() <= 1 then
    QuestFrameCompleteQuestButton:Click()
  end
end

function QuestingModule:QUEST_PROGRESS()
  self:Debug('QuestingModule:QUEST_PROGRESS')

  if not self:shouldAutomate() then return end
  if not self:shouldComplete() then return end
  if IsQuestCompletable() then
    CompleteQuest()
  end
end

function QuestingModule:QUEST_LOG_UPDATE()
  self:Debug('QuestingModule:QUEST_LOG_UPDATE')

  local startEntry = GetQuestLogSelection()
  local numEntries = GetNumQuestLogEntries()
  local title
  local isComplete
  local noObjectives
  local _

  self.completedQuests = {}
  self.incompleteQuests = {}

  self:Debug('QuestingModule:QUEST_LOG_UPDATE:startEntry', startEntry)
  self:Debug('QuestingModule:QUEST_LOG_UPDATE:numEntries', numEntries)

  if numEntries > 1 then
    for i = 1, numEntries do
      SelectQuestLogEntry(i)
      title, _, _, _, _, isComplete = GetQuestLogTitle(i)

  self:Debug('QuestingModule:QUEST_LOG_UPDATE:questLogEntry', i, isComplete, title)

      if title then
        if isComplete then
          self.completedQuests[title] = true
        else
          self.incompleteQuests[title] = true
        end
      end
    end
  end

  SelectQuestLogEntry(startEntry)
end

function QuestingModule:GOSSIP_SHOW()
  self:Debug('QuestingModule:GOSSIP_SHOW')

  if not self:shouldAutomate() then return end

  local button
  local text

  if
    GossipTitleButton1:IsVisible() and
    (
      GossipTitleButton1.type == 'Gossip' or
      GossipTitleButton1.type == 'Active'
    ) and
    not GossipTitleButton2:IsVisible()
  then
    GossipTitleButton1:Click()
    return
  end

  for i = 32, 1, -1 do
    button = _G['GossipTitleButton' .. i]

    if button:IsVisible() then
  self:Debug('QuestingModule:GOSSIP_SHOW:button', i)

      text = self:stripText(button:GetText())

  self:Debug('QuestingModule:GOSSIP_SHOW:button:text', i, text)
  self:Debug('QuestingModule:GOSSIP_SHOW:button:type', i, button.type)

      if button.type == 'Available' and self:shouldAccept() then
        button:Click()
      end

  self:Debug('QuestingModule:GOSSIP_SHOW:button:completedQuests', i, self.completedQuests[i])

      if
        button.type == 'Active' and
        self:shouldComplete() and
        self.completedQuests[text]
      then
        button:Click()
      end
    end
  end
end

function QuestingModule:QUEST_GREETING(...)
  self:Debug('QuestingModule:QUEST_GREETING')

  if not self:shouldAutomate() then return end

  local button
  local text

  for i = 32, 1, -1 do
    button = _G['QuestTitleButton' .. i]

    if button and button:IsVisible() then
      text = self:stripText(button:GetText())
      if
        self.completedQuests[text] and
        self:shouldComplete()
      then
        button:Click()
      elseif
        not self.incompleteQuests[text] and
        self:shouldAccept()
      then
        button:Click()
      end
    end
  end
end

function QuestingModule:QUEST_DETAIL()
  self:Debug('QuestingModule:QUEST_DETAIL')

  if not self:shouldAutomate() then return end
  if not self:shouldAccept() then return end

  AcceptQuest()
end
